class GroundPicturesController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user, :only => [:new, :edit, :create, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :edit, :create, :update, :destroy]
  else
    before_filter :authenticate_reader, :only => [:index, :show]
    before_filter :authenticate_user, :only => [:new, :edit, :create, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :edit, :create, :update, :destroy]
  end

  def index
    @page = GroundPicture.page params[:page]
    @page_size = GroundPicture.page_size params[:page_size]
    @ground_pictures = GroundPicture.list(@page, @page_size)

    respond_to do |format|
      format.html {
        @paginate = GroundPicture.list_paginate(@page, @page_size)
      }
      format.json { render :json => @ground_pictures.to_json(GroundPicture.list_json_opt) }
    end
  end

  def show
    @ground_picture = GroundPicture.show(params[:id], [@user, @admin])
    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @ground_picture.to_json(GroundPicture.show_json_opt) }
    end
  end
  
  def new
    raise Pettanr::NotWork unless @author.working_panel
    @picture = Picture.show params[:picture_id], @author
    raise ActiveRecord::Forbidden unless @picture.enable?
    @panel = Panel.edit(@author.working_panel, @author)
    
    @ground_picture = GroundPicture.new :panel_id => @panel.id, :picture_id => @picture.id
    @ground_picture.supply_default

    respond_to do |format|
      format.html
      format.json { render :json => @ground_picture.to_json(GroundPicture.show_json_opt) }
    end
  end

  def edit
    @ground_picture = GroundPicture.show(params[:id], @author)
    @panel = Panel.edit(@ground_picture.panel.id, @author)
    
    respond_to do |format|
      format.html
    end
  end

  def create
    raise Pettanr::NotWork unless @author.working_panel
    @panel = Panel.edit(@author.working_panel, @author)
    
    @ground_picture = GroundPicture.new
    @ground_picture.attributes = params[:ground_picture]
    @ground_picture.overwrite @panel.id
    
    @picture = Picture.show @ground_picture.picture_id, @author
    raise ActiveRecord::Forbidden unless @picture.enable?
    
    respond_to do |format|
      if @ground_picture.valid?
        if @ground_picture.store @author
          flash[:notice] = I18n.t('flash.notice.created', :model => Panel.model_name.human)
          format.html { redirect_to @panel }
          format.json { render json: @panel.panel_elements_as_json, status: :created, location: @panel }
        else
          flash[:notice] = I18n.t('flash.notice.not_created', :model => GroundPicture.model_name.human)
          format.html { render action: "new" }
          format.json { render json: @panel.errors, status: :unprocessable_entity }
        end
      else
        flash[:notice] = I18n.t('flash.notice.not_created', :model => GroundPicture.model_name.human)
        format.html { render action: "new" }
        format.json { render json: @ground_picture.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @ground_picture = GroundPicture.show(params[:id], @author)
    @ground_picture.attributes = params[:ground_picture]
    @panel = Panel.edit(@ground_picture.panel.id, @author)
    @ground_picture.overwrite @panel.id
    
    @picture = Picture.show @ground_picture.picture_id, @author
    raise ActiveRecord::Forbidden unless @picture.enable?
    
    respond_to do |format|
      if @ground_picture.store @author
        flash[:notice] = I18n.t('flash.notice.updated', :model => GroundPicture.model_name.human)
        format.html { redirect_to @ground_picture }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_updated', :model => GroundPicture.model_name.human)
        format.html { render action: "edit" }
        format.json { render json: @ground_picture.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @ground_picture = GroundPicture.show(params[:id], @author)
    @panel = Panel.edit(@ground_picture.panel.id, @author)
    
    respond_to do |format|
      if @ground_picture.remove @author
        flash[:notice] = I18n.t('flash.notice.destroyed', :model => GroundPicture.model_name.human)
        format.html { redirect_to @panel }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_destroyed', :model => GroundPicture.model_name.human)
        format.html { redirect_to @ground_picture }
        format.json { render json: @ground_picture.errors, status: :unprocessable_entity }
      end
    end
  end
  
end
