class StoriesController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  else
    before_filter :authenticate_reader, :only => [:index, :show, :play]
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  end
  before_filter :authenticate_admin!, :only => [:list, :browse]

  def index
    @page = Story.page params[:page]
    @page_size = Story.page_size params[:page_size]
    @stories = Story.list(@page, @page_size)

    respond_to do |format|
      format.html {
        @paginate = Story.list_paginate(@page, @page_size)
      }
      format.json { render :json => @stories.to_json(Story.list_json_opt) }
    end
  end

  def show
    @story = Story.show(params[:id], [@user, @admin])

    respond_to do |format|
      format.html { 
        if @author
          @new_sheets = Sheet.mylist(@author, 1, 5)
        end
      }
      format.json { render json: @story.to_json(Story.show_json_opt) }
    end
  end
  
  def play
    @story = Story.show(params[:id], [@user, @admin])
    @page = StorySheet.page params[:page]
    @sheet = StorySheet.play_sheet(@story, @author, @page)
    respond_to do |format|
      format.html {
        @paginate = StorySheet.play_paginate(@story, @page)
        if @author
          @new_sheets = Sheet.mylist(@author, 1, 5)
        end
      }
      format.json {
        if @sheet
          @story_sheets = SheetPanel.play_list @sheet, @author
          render text: SheetPanel.list_as_json_text(@story_sheets, @author)
        else
          render text: ''
        end
      }
      format.jsonp {
        render :json => "callback(" + @story_sheets.to_json_list + ");"
      }
    end
  end
  
  def list
    @stories = Story.all

    respond_to do |format|
      format.html { render layout: 'system' }# index.html.erb
      format.json { render json: @stories }
    end
  end

  def browse
    @story = Story.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' } # show.html.erb
      format.json { render json: @story }
    end
  end
  
  def new
    @story = Story.new 
    @story.supply_default
    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @story.to_json(Story.show_json_opt) }
    end
  end

  def edit
    @story = Story.edit(params[:id], @author)
    respond_to do |format|
      format.html 
      format.js
    end
  end

  def create
    @story = Story.new 
    @story.supply_default
    @story.attributes = params[:story]
    @story.overwrite
    @comic = Comic.edit(@story.comic_id, @author) if @story.comic_id
    
    respond_to do |format|
      if @story.store @author
        flash[:notice] = I18n.t('flash.notice.created', :model => Story.model_name.human)
        format.html { redirect_to play_story_path(@story, :page => @story.t.to_i + 1) }
        format.json { render json: @story.to_json(Story.show_json_opt) }
      else
        flash[:notice] = I18n.t('flash.notice.not_created', :model => Story.model_name.human)
        format.html { render action: "new" }
        format.json { render json: @story.errors, status: :unprocessable_entity }
      end
    end
  end
  
  def update
    @story = Story.edit(params[:id], @author)
    ot = @story.t
    @story.attributes = params[:story]
    @story.overwrite
    respond_to do |format|
      if @story.store  @author, ot
        flash[:notice] = I18n.t('flash.notice.updated', :model => Story.model_name.human)
        format.html { redirect_to play_story_path(@story, :page => @story.t.to_i + 1) }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_updated', :model => Story.model_name.human)
        format.html { render action: "edit" }
        format.json { render json: @story.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @story = Story.edit(params[:id], @author)
    respond_to do |format|
      if @story.destroy_and_shorten
        flash[:notice] = I18n.t('flash.notice.destroyed', :model => Story.model_name.human)
        format.html { redirect_to comic_path(@story.comic)}
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_destroyed', :model => Story.model_name.human)
        format.html { redirect_to story_path(@story) }
        format.json { render json: @story.errors, status: :unprocessable_entity }
      end
    end
  end
end
