class Sheet < ActiveRecord::Base
  has_many :sheet_panels
  has_many :story_sheets
  belongs_to :author
  
  validates :caption, :presence => true, :length => {:maximum => 100}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :visible, :presence => true, :numericality => true, :inclusion => {:in => 0..1}
  accepts_nested_attributes_for :sheet_panels, :allow_destroy => true
  before_validation :valid_encode
  
  def valid_encode
    ['caption'].each do |a|
      next if attributes[a] == nil
      raise Pettanr::BadRequest unless attributes[a].valid_encoding?
    end
  end
  
  def self.each_element_class_names
    {'SheetPanel' => 'sheet_panels'}.each do |k, n|
      yield k
    end
  end
  
  def self.class_name_to_class k
    Object.const_get k
  end
  
  def self.each_element_classes
    self.each_element_class_names do |k|
      e = self.class_name_to_class k
      yield e
    end
  end
  
  def elements_by_class_name class_name
    self.__send__ class_name.tableize
  end
  
  def supply_default
    self.visible = 0 if self.visible.blank?
  end
  
  def overwrite au
    return false unless au
    self.author_id = au.id
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    au = Sheet.get_author_from_roles roles
    return false unless au
    self.author_id == au.id
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.own?(roles)
    return true if self.new_record?
    self.visible > 0
  end
  
  def usable? au
    visible? au
  end
  
  def tag_id
    'sheet' + self.tag_sheet_id
  end
  
  def tag_sheet_id
    self.new_record? ? '0' : self.id.to_s
  end
  
  def field_tag_id f
    self.tag_id + f.to_s
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list_where
    'sheets.visible > 0'
  end
  
  def self.mylist_where au
    ['sheets.author_id = ?', au.id]
  end
  
  def self.himlist_where au
    ['sheets.author_id = ? and sheets.visible > 0', au.id]
  end
  
  def self.list page = 1, page_size = self.default_page_size
    Sheet.where(self.list_where()).includes(Sheet.list_opt).order('sheets.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.mylist au, page = 1, page_size = Author.default_sheet_page_size
    Sheet.where(self.mylist_where(au)).includes(Sheet.list_opt).order('sheets.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.himlist au, page = 1, page_size = Author.default_sheet_page_size
    Sheet.where(self.himlist_where(au)).includes(Sheet.list_opt).order('sheets.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.list_paginate page = 1, page_size = self.default_page_size
    Kaminari.paginate_array(Array.new(Sheet.where(self.list_where()).count, nil)).page(page).per(page_size)
  end
  
  def self.mylist_paginate au, page = 1, page_size = Author.default_sheet_page_size
    Kaminari.paginate_array(Array.new(Sheet.where(self.mylist_where(au)).count, nil)).page(page).per(page_size)
  end
  
  def self.himlist_paginate au, page = 1, page_size = Author.default_sheet_page_size
    Kaminari.paginate_array(Array.new(Sheet.where(self.himlist_where(au)).count, nil)).page(page).per(page_size)
  end
  
  def self.list_opt
    {:sheet_panels => {:panel => {}}, :author => {} }
  end
  
  def self.list_json_opt
    {:include => {:sheet_panels => {:include => {:panel => {}}}, :author => {}}}
  end
  
  def self.show sid, roles
    opt = {}
    opt.merge!(Sheet.show_opt)
    res = Sheet.find(sid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(roles)
    res
  end
  
  def self.edit sid, au
    opt = {}
    opt.merge!(Sheet.show_opt)
    res = Sheet.find(sid, opt)
    raise ActiveRecord::Forbidden unless res.own?(au)
    res
  end
  
  def self.show_opt
    {:include => {:sheet_panels => {:panel => {}}, :author => {}}}
  end
  
  def self.show_json_opt
    {:include => {:sheet_panels => {:include => {:panel => {}}}, :author => {}}}
  end
  
  def parts_element
    r = []
    self.class.each_element_class_names do |k|
      r += (self.elements_by_class_name(k) || [])
    end
    r
  end
  
  def zorderd_elements
    res = []
    self.parts_element.each do |e|
      res[e.z-1] = e
    end
    res
  end
  
  def sheet_elements
    res = []
    self.parts_element.each do |e|
      res[e.t] = e
    end
    res
  end
  
  def new_t
    self.sheet_elements.size
  end
  
  def new_z
    self.sheet_elements.size + 1
  end
  
  def self.visible_count
    Sheet.count 'visible > 0'
  end
  
  def self.collect_element_value elements, name
    elements.map {|e|
      e.map {|o|
        if o['_destroy'] or o[:_destroy]
          nil
        else
          o[name]
        end
      }.compact
    }.flatten
  end
  
  def self.validate_serial ary, offset = 0
    i = offset
    ary.compact.sort.each do |n|
      break false unless n == i
      i += 1
    end
    ary.compact.size == i - offset
  end
  
  def self.validate_element_serial elements, name, offset = 0
    Sheet.validate_serial(Sheet.collect_element_value(elements, name), offset)
  end
  
  def self.validate_elements_serial c
    c.map {|conf|
      Sheet.validate_element_serial(conf[:elements], conf[:name], conf[:offset]) ? nil : false
    }.compact.empty?
  end
  
  def validate_serial_list
    l = []
    self.class.each_element_class_names do |k|
      l << self.elements_by_class_name(k)
    end
#    l = [self.sheet_panels]
    [
      {:elements => l, :name => :t, :offset => 0}, 
      {:elements => l, :name => :z, :offset => 1}
    ]
  end
  
  def validate_child
#    r1 = Panel.validate_elements_id validate_id_list
    Sheet.validate_elements_serial validate_serial_list
  end
  
  def boost
    @new_element_index = 0
    self.panel_elements.each do |elm|
      if elm.new_record?
        elm.new_index = @new_element_index
        @new_element_index += 1
      end
    end
  end 
  
  def store attr, au
    if attr == false
      self.errors.add :base, I18n.t('errors.invalid_json')
      return false
    end
    self.attributes = attr
    self.overwrite au
    res = false
    Sheet.transaction do
      self.panel_elements.each do |elm|
        elm.new_sheet = self
      end
      res = self.save
      unless validate_child
        res = false
        self.errors.add :base, I18n.t('errors.invalid_t')
        raise ActiveRecord::Rollback
      end
    end
    res
  end
  
  def destroy_with_sheet_panel
    res = false
    Sheet.transaction do
      self.sheet_panels.each do |sheet_panel|
        raise ActiveRecord::Rollback unless sheet_panel.destroy
      end
      raise ActiveRecord::Rollback unless self.destroy
      res = true
    end
    res
  end
  
  def scenario
    panels.map {|panel|
      panel.scenario
    }.join
  end
  
  def plain_scenario
    panels.map {|panel|
      panel.plain_scenario
    }.join
  end
  
  def licensed_pictures
    r = {}
    self.sheet_elements.each do |elm|
      r.merge!(elm.licensed_pictures)
    end
    r
  end
  
  def copy
    attr = self.copy_attributes
    Sheet.each_element_class_names do |n|
      attr.merge! Sheet.class_name_to_class(n).panelize(self.elements_by_class_name(n).map {|elm|  elm.copy_attributes})
    end
    attr
  end
  
  def copy_attributes
    r = self.attributes
    r.delete 'id'
    r.delete 'author_id'
    r.delete 'created_at'
    r.delete 'updated_at'
    r
  end
  
  def self.panelize sheet
    attr = sheet.attributes
    attr.delete 'id'
    attr
  end
  
end
