# -*- encoding: utf-8 -*-
require 'spec_helper'
#色地

describe GroundColor do
  before do
    SpeechBalloonTemplate.delete_all
    @admin = FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = FactoryGirl.create :author, :user_id => @user.id
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = FactoryGirl.create :author, :user_id => @other_user.id
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
    @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
    @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
    @panel = FactoryGirl.create :panel, :author_id => @author.id
  end
  
  describe '検証に於いて' do
    before do
      @gc = FactoryGirl.build :ground_color, :panel_id => @panel.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @gc.code = 0
        @gc.z = 1
        @gc.t = 0
        @gc.should be_valid
      end
      it '上限データが通る' do
        @gc.code = 99999
        @gc.z = 99999
        @gc.t = 99999
        @gc.should be_valid
      end
    end
    
    context 'panel_idを検証するとき' do
      #ネストの保存はnilを許可しなければならないので数値チェックだけ
      it '数値でなければ失敗する' do
        @gc.panel_id = 'a'
        @gc.should_not be_valid
      end
    end
    context 'codeを検証するとき' do
      it 'テストデータの確認' do
        @gc.code = 0xffffff
        @gc.should be_valid
      end
      it 'nullなら失敗する' do
        @gc.code = nil
        @gc.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @gc.code = 'a'
        @gc.should_not be_valid
      end
      it '負なら失敗する' do
        @gc.code = -1
        @gc.should_not be_valid
      end
      it '24bit colorでなければ失敗する' do
        @gc.code = 0x1000000
        @gc.should_not be_valid
      end
    end
    context 'zを検証するとき' do
      it 'nullなら失敗する' do
        @gc.z = nil
        @gc.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @gc.z = 'a'
        @gc.should_not be_valid
      end
      it '負なら失敗する' do
        @gc.z = -1
        @gc.should_not be_valid
      end
      it '負なら失敗する' do
        @gc.z = 0
        @gc.should_not be_valid
      end
    end
    context 'tを検証するとき' do
      it 'nullなら失敗する' do
        @gc.t = nil
        @gc.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @gc.t = 'a'
        @gc.should_not be_valid
      end
      it '負なら失敗する' do
        @gc.t = -1
        @gc.should_not be_valid
      end
    end
  end
  
  describe '文字コード検証に於いて' do
    before do
      @gc = FactoryGirl.build :ground_color, :panel_id => @panel.id
    end
    
    context 'captionを検証するとき' do
      it 'Shift JISなら失敗する' do
        @gc.caption = "\x83G\x83r\x83]\x83D"
        lambda{
          @gc.valid_encode
        }.should raise_error(Pettanr::BadRequest)
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'defined' do
      @gc = FactoryGirl.build :ground_color, :panel_id => @panel.id
      @gc.supply_default
    end
  end
  
  describe '上書き補充に於いて' do
    it 'panel_idが設定されている' do
      @gc = FactoryGirl.build :ground_color, :panel_id => @panel.id
      @gc.overwrite  @panel.id
      @gc.panel_id.should eq @panel.id
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @gc = FactoryGirl.create :ground_color, :panel_id => @panel.id
    end
    context 'オープンモードのとき' do
      before do
        MagicNumber['run_mode'] = 0
      end
      it '自身にゲスト用ロールチェックを問い合わせしている' do
        GroundColor.any_instance.stub(:guest_role_check).and_return(true)
        GroundColor.any_instance.should_receive(:guest_role_check).with(any_args).exactly(1)
        r = @gc.visible?([@author])
      end
      it 'ゲスト用ロールチェックが失敗したとき、falseを返す' do
        GroundColor.any_instance.stub(:guest_role_check).and_return(false)
        r = @gc.visible?([@author])
        r.should be_false
      end
    end
    context 'クローズドモードのとき' do
      before do
        MagicNumber['run_mode'] = 1
      end
      it '自身に読者用ロールチェックを問い合わせしている' do
        GroundColor.any_instance.stub(:reader_role_check).and_return(true)
        GroundColor.any_instance.should_receive(:reader_role_check).with(any_args).exactly(1)
        r = @gc.visible?([@author])
      end
      it '読者用ロールチェックが失敗したとき、falseを返す' do
        GroundColor.any_instance.stub(:reader_role_check).and_return(false)
        r = @gc.visible?([@author])
        r.should be_false
      end
    end
    context '事前チェックする' do
      before do
        MagicNumber['run_mode'] = 1
        GroundColor.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自身のコマに所持判定を問い合わせしている' do
        Panel.any_instance.stub(:own?).and_return(true)
        Panel.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = @gc.visible?([@author])
      end
      it '自身のコマに閲覧許可を問い合わせしている' do
        Panel.any_instance.stub(:own?).and_return(false)
        Panel.any_instance.stub(:visible?).and_return(true)
        Panel.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = @gc.visible?([@author])
      end
    end
    context 'つつがなく終わるとき' do
      before do
        MagicNumber['run_mode'] = 1
        Panel.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自分のコマの色地なら許可する' do
        Panel.any_instance.stub(:own?).and_return(true)
        Panel.any_instance.stub(:visible?).and_return(false)
        r = @gc.visible?([@author])
        r.should be_true
      end
      it '他人の非公開コマの色地なら許可しない' do
        Panel.any_instance.stub(:own?).and_return(false)
        Panel.any_instance.stub(:visible?).and_return(false)
        r = @gc.visible?([@author])
        r.should be_false
      end
      it '他人のコマの色地でも公開なら許可する' do
        Panel.any_instance.stub(:own?).and_return(false)
        Panel.any_instance.stub(:visible?).and_return(true)
        r = @gc.visible?([@author])
        r.should be_true
      end
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @gc = FactoryGirl.create :ground_color, :panel_id => @panel.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        GroundColor.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        GroundColor.page().should eq 1
      end
      it '0以下の場合は1になる' do
        GroundColor.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        GroundColor.page_size('7').should eq 7
      end
      it 'nilの場合はGroundColor.default_page_sizeになる' do
        GroundColor.page_size().should eq GroundColor.default_page_size
      end
      it '0以下の場合はGroundColor.default_page_sizeになる' do
        GroundColor.page_size('0').should eq GroundColor.default_page_size
      end
      it 'GroundColor.max_page_sizeを超えた場合はGroundColor.max_page_sizeになる' do
        GroundColor.page_size('1000').should eq GroundColor.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        GroundColor.stub(:list_opt).with(any_args).and_return({:include => :panel})
        GroundColor.should_receive(:list_opt).with(any_args).exactly(1)
        r = GroundColor.list
      end
    end
    it 'リストを返す' do
      pl = GroundColor.list
      pl.should eq [@gc]
    end
    it '時系列で並んでいる' do
      #公開コマなら(他人のコマであっても)含んでいる
      opl = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      npl = FactoryGirl.create :ground_color, :panel_id => opl.id, :updated_at => Time.now + 100
      pl = GroundColor.list
      pl.should eq [npl, @gc]
    end
    it '非公開のコマの色地は含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      npl = FactoryGirl.create :ground_color, :panel_id => hpl.id, :updated_at => Time.now + 100
      pl = GroundColor.list
      pl.should eq [@gc]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @gc2 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 1, :updated_at => Time.now + 100
        @gc3 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 2, :updated_at => Time.now + 200
        @gc4 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 3, :updated_at => Time.now + 300
        @gc5 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 4, :updated_at => Time.now + 400
        GroundColor.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        pl = GroundColor.list
        pl.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pl = GroundColor.list(1)
        pl.should eq [@gc5, @gc4]
      end
      it 'page=2なら中間2件を返す' do
        pl = GroundColor.list(2)
        pl.should eq [@gc3, @gc2]
      end
      it 'page=3なら先頭1件を返す' do
        pl = GroundColor.list(3)
        pl.should eq [@gc]
      end
    end
  end
  
  describe '自分のコマで使った色地一覧取得に於いて' do
    before do
      @gc = FactoryGirl.create :ground_color, :panel_id => @panel.id
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        GroundColor.stub(:list_opt).with(any_args).and_return({:include => :panel})
        GroundColor.should_receive(:list_opt).with(any_args).exactly(1)
        r = GroundColor.mylist @author
      end
    end
    it 'リストを返す' do
      pl = GroundColor.mylist @author
      pl.should eq [@gc]
    end
    it '時系列で並んでいる' do
      npl = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 2, :updated_at => Time.now + 100
      pl = GroundColor.mylist @author
      pl.should eq [npl, @gc]
    end
    it '他人のコマの色地は公開でも含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      npl = FactoryGirl.create :ground_color, :panel_id => hpl.id
      pl = GroundColor.mylist @author
      pl.should eq [@gc]
    end
    it '自分のコマの色地は非公開でも含んでいる' do
      hpl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      npl = FactoryGirl.create :ground_color, :panel_id => hpl.id, :z => 2, :updated_at => Time.now + 100
      pl = GroundColor.mylist @author
      pl.should eq [npl, @gc]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @gc2 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 1, :updated_at => Time.now + 100
        @gc3 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 2, :updated_at => Time.now + 200
        @gc4 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 3, :updated_at => Time.now + 300
        @gc5 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 4, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        c = GroundColor.mylist @author, 1, 2
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = GroundColor.mylist(@author, 1, 2)
        c.should eq [@gc5, @gc4]
      end
      it 'page=2なら中間2件を返す' do
        c = GroundColor.mylist(@author, 2, 2)
        c.should eq [@gc3, @gc2]
      end
      it 'page=3なら先頭1件を返す' do
        c = GroundColor.mylist(@author, 3, 2)
        c.should eq [@gc]
      end
    end
  end
  
  describe '他作家の色地一覧取得に於いて' do
    before do
      @gc = FactoryGirl.create :ground_color, :panel_id => @panel.id
      @other_panel = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      @other_gc = FactoryGirl.create :ground_color, :panel_id => @other_panel.id
    end
    it 'リストを返す' do
      r = GroundColor.himlist @other_author
      r.should eq [@other_gc]
    end
    it '時系列で並んでいる' do
      new_panel = FactoryGirl.create :panel, :author_id => @other_author.id, :updated_at => Time.now + 100
      new_gc = FactoryGirl.create :ground_color, :panel_id => @other_panel.id, :updated_at => Time.now + 100
      r = GroundColor.himlist @other_author
      r.should eq [new_gc, @other_gc]
    end
    it '公開コマに限る' do
      hidden_panel = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 0
      hidden_gc = FactoryGirl.create :ground_color, :panel_id => hidden_panel.id
      r = GroundColor.himlist @other_author
      r.should eq [@other_gc]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @other_gc2 = FactoryGirl.create :ground_color, :panel_id => @other_panel.id, :updated_at => Time.now + 100
        @other_gc3 = FactoryGirl.create :ground_color, :panel_id => @other_panel.id, :updated_at => Time.now + 200
        @other_gc4 = FactoryGirl.create :ground_color, :panel_id => @other_panel.id, :updated_at => Time.now + 300
        @other_gc5 = FactoryGirl.create :ground_color, :panel_id => @other_panel.id, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        pl = GroundColor.himlist @other_author, 1, 2
        pl.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pl = GroundColor.himlist @other_author, 1, 2
        pl.should eq [@other_gc5, @other_gc4]
      end
      it 'page=2なら中間2件を返す' do
        pl = GroundColor.himlist @other_author, 2, 2
        pl.should eq [@other_gc3, @other_gc2]
      end
      it 'page=3なら先頭1件を返す' do
        pl = GroundColor.himlist @other_author, 3, 2
        pl.should eq [@other_gc]
      end
    end
  end
  
  describe '色地一覧ページ制御に於いて' do
    before do
      GroundColor.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = GroundColor.list_paginate 
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it '色地一覧の取得条件を利用している' do
      GroundColor.stub(:list_where).with(any_args).and_return('')
      GroundColor.should_receive(:list_where).with(any_args).exactly(1)
      r = GroundColor.list_paginate 
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = GroundColor.list_paginate 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '自分の色地一覧ページ制御に於いて' do
    before do
      GroundColor.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = GroundColor.mylist_paginate @author
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it '自分の色地一覧の取得条件を利用している' do
      GroundColor.stub(:mylist_where).with(any_args).and_return('')
      GroundColor.should_receive(:mylist_where).with(any_args).exactly(1)
      r = GroundColor.mylist_paginate @author
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = GroundColor.mylist_paginate @author, 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '他作家の色地一覧ページ制御に於いて' do
    before do
      GroundColor.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = GroundColor.himlist_paginate @other_author
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it '他作家の色地一覧の取得条件を利用している' do
      GroundColor.stub(:himlist_where).with(any_args).and_return('')
      GroundColor.should_receive(:himlist_where).with(any_args).exactly(1)
      r = GroundColor.himlist_paginate @other_author
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = GroundColor.himlist_paginate @other_author, 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '一覧取得オプションに於いて' do
    it '1つの項目を含んでいる' do
      r = GroundColor.list_opt
      r.should have(1).items
    end
    it 'コマを含んでいる' do
      r = GroundColor.list_opt
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = GroundColor.list_opt
        r[:panel].has_key?(:author).should be_true
      end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
      @sbt = FactoryGirl.create :speech_balloon_template
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id, :visible => 1
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
      @gc = FactoryGirl.create :ground_color, :panel_id => @panel.id
    end
    it 'コマを含んでいる' do
      r = GroundColor.list.to_json GroundColor.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('panel').should be_true
    end
      it 'コマは作家を含んでいる' do
        r = GroundColor.list.to_json GroundColor.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['panel']
        s.has_key?('author').should be_true
      end
  end
  
  describe '単体取得に於いて' do
    before do
      @gc = FactoryGirl.create :ground_color, :panel_id => @panel.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        GroundColor.stub(:show_opt).with(any_args).and_return({:include => :panel})
        GroundColor.should_receive(:show_opt).with(any_args).exactly(1)
        r = GroundColor.show @gc.id, @author
      end
      it '閲覧許可を問い合わせている' do
        GroundColor.any_instance.stub(:visible?).with(any_args).and_return(true)
        GroundColor.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = GroundColor.show @gc.id, @author
      end
    end
    it '指定の色地を返す' do
      GroundColor.any_instance.stub(:visible?).and_return(true)
      pl = GroundColor.show @gc.id, @author
      pl.should eq @gc
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        GroundColor.any_instance.stub(:visible?).and_return(false)
        lambda{
          GroundColor.show @gc.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しない色地を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          GroundColor.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = GroundColor.show_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = GroundColor.show_opt[:include]
      r.should have(1).items
    end
    it 'コマを含んでいる' do
      r = GroundColor.show_opt[:include]
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = GroundColor.show_opt[:include]
        r[:panel].has_key?(:author).should be_true
      end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @gc = FactoryGirl.create :ground_color, :panel_id => @panel.id
    end
    it 'コマを含んでいる' do
      r = GroundColor.show(@gc.id, @author).to_json GroundColor.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('panel').should be_true
    end
      it 'コマは作家を含んでいる' do
        r = GroundColor.show(@gc.id, @author).to_json GroundColor.show_json_opt
        j = JSON.parse r
        i = j
        s = i['panel']
        s.has_key?('author').should be_true
      end
  end
  
end
